package winMain;

#
# winMain.pm
#
# [ldasm] The main-window
#

use strict;
use Tk;
use Tk::Text;
use Tk::Balloon;

use config;
use winFunc;
use misc;
use asmps;
#-------
# Export

use Exporter;
use vars qw(@ISA @EXPORT $win $text $currentRow);
@ISA = qw(Exporter);
@EXPORT = qw(&CreateWinMain);

#-----------------
# Global variables

my ($currentOfs);
$currentRow = 1;
$currentOfs = 1;
my $winBalloon;

#--------------------
# sub CreateWinMain()
#
# Creates the main-window

sub CreateWinMain()
{
   $win = MainWindow->new();
   $win->title($config::WinTitle);
   $win->iconname($config::IconName);

   # Ballon for the main-window (tooltips in Windows)
   $winBalloon = $win->Balloon(
      -initwait => 400,
      -state    => 'balloon'
   );

   # Bindings
   $win->bind('<Up>'    => sub { MoveRowSelection('-1'); });
   $win->bind('<Down>'  => sub { MoveRowSelection('+1'); });
   $win->bind('<Prior>' => sub { MoveRowSelection('-p'); });
   $win->bind('<Next>'  => sub { MoveRowSelection('+p'); });

   # Add menu
   my $menuFrame = $win->Frame()->pack(
      -side => 'top',
      -fill => 'x'
   );

   # Shared entries btw. Menubuttons and Toolbar-buttons
   my %shareMenu = (
      'open'      => { num => 0,  cmd => \&OpenFile,
                       str => 'Open File to Disassemble...'},
      'save'      => { num => 1,  cmd => \&SavePrj,
                       str => 'Save Disassembly Text File'},
      'search'    => { num => 2,  cmd => \&SearchText,
                       str => 'Find Text'},
      'copy'      => { num => 3,  cmd => \&CopySelCode,
                       str => 'Copy Selected Line'},
      'gotocode'  => { num => 4,  cmd => [\&GotoCodePoint,'start'],
                       str => 'Goto Code Start'},
      'gotoentry' => { num => 5,  cmd => [\&GotoCodePoint,'ep'],
                       str => 'Goto Program Entry Point'},
      'gotopage'  => { num => 6,  cmd => undef,
                       str => 'Goto Page'},
      'gotoloc'   => { num => 7,  cmd => \&GotoCodeLoc,
                       str => 'Goto Code Location'},
      'jump'      => { num => 8,  cmd => [\&ExecuteRefOp,'jump'],
                       str => 'Execute Jump'},
      'retjmp'    => { num => 9,  cmd => [\&ExecuteRefRet,'jump'],
                       str => 'Return From Last Jump'},
      'call'      => { num => 10, cmd => [\&ExecuteRefOp,'call'],
                       str => 'Execute Call'},
      'retcall'   => { num => 11, cmd => [\&ExecuteRefRet,'call'],
                       str => 'Return From Call'},
      'functions' => { num => 12, cmd => \&FuncImport,
                       str => 'Imports'},
      'exports'   => { num => 13, cmd => \&FuncExport,
                       str => 'Exports'},
      'internal'  => { num => 14, cmd => \&FuncInternal,
                       str => 'Internals'},
      'hexdata'   => { num => 15, cmd => undef,
                       str => 'Hex Display of Data Object/Segments'},
      'hexcode'   => { num => 16, cmd => undef,
                       str => 'Hex Display of Code Data'},
      'menuref'   => { num => 17, cmd => undef,
                       str => 'Menu References'},
      'dialogref' => { num => 18, cmd => undef,
                       str => 'Dialog References'},
      'strings'   => { num => 19, cmd => \&StringRefs,
                       str => 'String Data References'},
      'print'     => { num => 20, cmd => undef,
                       str => 'Print...'}
   );

   # Menu:Disassembler
   my $db = $menuFrame->Menubutton(
      -text      => 'Disassembler',
      -menuitems => [
         [ command   => $shareMenu{'open'}->{str},
          -command   => $shareMenu{'open'}->{cmd},
          -underline => 0],
         '-',
         [ command   => $shareMenu{'save'}->{str},
          -command   => $shareMenu{'save'}->{cmd},
          -underline => 0],
         '-',
         [ command   => 'Print Preview',
          -command   => undef,
          -underline => 0,
          -state     => 'disabled'],
         [ command   => $shareMenu{'print'}->{str},
          -command   => $shareMenu{'print'}->{cmd},
          -underline => 0,
          -state     => 'disabled'],
         [ command   => 'Print Setup...',
          -command   => undef,
          -underline => 0,
          -state     => 'disabled'],
         '-',
         [ command   => $shareMenu{'copy'}->{str},
          -command   => $shareMenu{'copy'}->{cmd},
          -underline => 0],
         '-',
         [ command   => 'Font...',
          -command   => undef,
          -underline => 0,
          -state     => 'disabled'],
         [ command   => 'Disassembler Options...',
          -command   => undef,
          -underline => 0,
          -state     => 'disabled'],
         '-',
         [ command   => 'Exit',
          -command   => sub {exit(0);},
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Project
   $menuFrame->Menubutton(
      -text      => 'Project',
      -menuitems => [
         [ command   => 'Open Disassembled File...',
          -command   => \&OpenPrj,
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Search
   $menuFrame->Menubutton(
      -text      => 'Search',
      -menuitems => [
         [ command   => $shareMenu{'search'}->{str},
          -command   => $shareMenu{'search'}->{cmd},
          -underline => 0],
         '-',
         [ command   => 'Find Next',
          -command   => \&SearchNext,
          -underline => 0],
         [ command   => 'Show result-window',
          -command   => \&SearchResultWin,
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Goto
   $menuFrame->Menubutton(
      -text      => 'Goto',
      -menuitems => [
         [ command   => $shareMenu{'gotocode'}->{str},
          -command   => $shareMenu{'gotocode'}->{cmd},
          -underline => 0],
         [ command   => $shareMenu{'gotoentry'}->{str},
          -command   => $shareMenu{'gotoentry'}->{cmd},
          -underline => 0],
         [ command   => $shareMenu{'gotopage'}->{str},
          -command   => $shareMenu{'gotopage'}->{cmd},
          -underline => 0,
          -state     => 'disabled'],
         [ command   => $shareMenu{'gotoloc'}->{str},
          -command   => $shareMenu{'gotoloc'}->{cmd},
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Execute
   $menuFrame->Menubutton(
      -text      => 'Execute Text',
      -menuitems => [
         [ command   => $shareMenu{'jump'}->{str},
          -command   => $shareMenu{'jump'}->{cmd},
          -underline => 0],
         [ command   => $shareMenu{'retjmp'}->{str},
          -command   => $shareMenu{'retjmp'}->{cmd},
          -underline => 0],
         '-',
         [ command   => $shareMenu{'call'}->{str},
          -command   => $shareMenu{'call'}->{cmd},
          -underline => 0],
         [ command   => $shareMenu{'retcall'}->{str},
          -command   => $shareMenu{'retcall'}->{cmd},
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Functions
   $menuFrame->Menubutton(
      -text      => 'Functions',
      -menuitems => [
         [ command   => $shareMenu{'functions'}->{str},
          -command   => $shareMenu{'functions'}->{cmd},
          -underline => 0],
         [ command   => $shareMenu{'exports'}->{str},
          -command   => $shareMenu{'exports'}->{cmd},
          -underline => 0],
         '-',
         [ command   => $shareMenu{'internal'}->{str},
          -command   => $shareMenu{'internal'}->{cmd},
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:HexData
   $menuFrame->Menubutton(
      -text      => 'HexData',
      -menuitems => [
         [ command   => $shareMenu{'hexdata'}->{str},
          -command   => $shareMenu{'hexdata'}->{cmd},
          -underline => 0,
          -state     => 'disabled'],
         [ command   => $shareMenu{'hexcode'}->{str},
          -command   => $shareMenu{'hexcode'}->{cmd},
          -underline => 0,
          -state     => 'disabled']
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Refs
   $menuFrame->Menubutton(
      -text      => 'Refs',
      -menuitems => [
         [ command   => $shareMenu{'menuref'}->{str},
          -command   => $shareMenu{'menuref'}->{cmd},
          -underline => 0,
          -state     => 'disabled'],
         [ command   => $shareMenu{'dialogref'}->{str},
          -command   => $shareMenu{'dialogref'}->{cmd},
          -underline => 0,
          -state     => 'disabled'],
         [ command   => $shareMenu{'strings'}->{str},
          -command   => $shareMenu{'strings'}->{cmd},
          -underline => 0]
         ]
      )->pack(
         -side => 'left',
         -padx => 2
   );

   # Menu:Help
   $menuFrame->Menubutton(
      -text      => 'Help',
      -menuitems => [
         [ command   => 'Content',
          -command   => \&HelpMe,
          -underline => 0],
         '-',
         [ command   => 'About...',
          -command   => \&About,
          -underline => 0]
         ]
      )->pack(
         -side => 'right',
         -padx => 2
   );

   # All the beautiful buttons (Toolbar)
   my $buttonFrame = $win->Frame()->pack(
      -side => 'top',
      -fill => 'x'
   );
   foreach(sort{$shareMenu{$a}->{num} <=> $shareMenu{$b}->{num}} keys %shareMenu) {
      my $bt = $buttonFrame->Button(
         -image   => $buttonFrame->Pixmap(-file => $config::RCPath."/icons/$_.xpm"),
         -command => $shareMenu{$_}->{cmd}
      )->pack(
         -side => 'left'
      );
      $winBalloon->attach($bt,
         -balloonmsg => $shareMenu{$_}->{str}
      );
   }

   # Statebar
   my $stateFrame = $win->Frame()->pack(
      -side => 'bottom',
      -fill => 'x'
   );

   # Statebar:Address
   $stateFrame->Label(
      -text => 'Row:'
   )->pack(
      -side => 'left'
   );
   $stateFrame->Label(
      -anchor       => 'w',
      -textvariable => \$currentRow,
      -relief       => 'sunken',
      -width        => '8'
   )->pack(
      -side => 'left'
   );

   # Statebar:Offset
   $stateFrame->Label(
      -text => 'Ofs:'
   )->pack(
      -side => 'left'
   );
   $stateFrame->Label(
      -anchor       => 'w',
      -textvariable => \$currentOfs,
      -relief       => 'sunken',
      -width        => '10'
   )->pack(
      -side => 'left'
   );

   # Widget for assembler-source+set tags+key bindings
   $text = $win->ScrlText(
      -state            => 'disabled',
      -scrollbars       => 'se',
      -wrap             => 'none',
      -takefocus        => 0,
      -bg               => 'gray',
      -selectforeground => 'black',
      -selectbackground => 'gray',
      -selectborderwidth => 0

   )->pack(
      -expand => 1,
      -fill   => 'both'
   );
   TextTags($text,%config::FontAsm);
}


1;
